/*
This is a Optical-Character-Recognition program
Copyright (C) 2000  Joerg Schulenburg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 Joerg.Schulenburg@physik.uni-magdeburg.de
 */

#include <stdio.h>
#include <stdlib.h>
#include "gocr.h"
#include "pnm.h"
#include "pgm2asc.h"
#include <string.h>

#define Blen  80

extern List boxlist;

// load database into boxd-chain
// this is added in version v0.2.4 
//   as alternate engine comparing chars with database
int load_db(void) {
  FILE *f1;
  char s1[Blen+1], s2[Blen+1] = "./db/";
  int i, j, ii, i2;
  struct box *box1;
  pix *pp;

    pp = (pix *)malloc(sizeof(pix));

  i2=strlen(s2);
  if (env.vvv)
    fprintf(stderr, "# load database %s ... ",s2);

  strncpy(s2+i2,"db.lst",Blen-i2);s2[Blen]=0;
  f1 = fopen(s2, "r");
  if (!f1) {
    fprintf(stderr, " DB %s not found\n",s2);
    return 1;
  }

  for (ii = 0; !feof(f1); ii++) {
/* bbg: should write a better input routine */
    fgets(s1, Blen, f1);
    j = strlen(s1);
    if (j == 0)
      continue;
    for (i = 0; i < j && i+i2 < Blen && strchr(" \t,;",s1[i]) == 0; i++)
      s2[i2 + i] = s1[i];
/* by now: read pix, fill box, goto next */
    pp = (pix *)malloc(sizeof(pix));
#ifdef HAVE_PNM_H
    readpnm(s2, pp, 0 * env.vvv);
#else
    readpgm(s2, pp, 0 * env.vvv);
#endif
//    box1 = (sBox *)malloc(sizeof(Box));
    box1->x0 = 1;
    box1->x1 = pp->x - 3;       // white border 1 pixel width
    box1->y0 = 1;
    box1->y1 = pp->y - 3;
    box1->x = 1;
    box1->y = 1;
    box1->dots = 0;
    box1->c = s1[i + 1];
    box1->ac = '_';
    box1->num = 0;
    box1->line = -1;
    box1->m1 = 0;
    box1->m2 = 0;
    box1->m3 = 0;
    box1->m4 = 0;
    box1->p = pp;
    list_app(&boxlist, box1);	// append to list
  }
  fclose(f1);
  if (env.vvv)
    fprintf(stderr, " %d chars loaded\n", ii);
  return 0;
}

// second variant, for database (with slightly other behaviour)
// new variant
//  look at the environment of the pixel too (contrast etc.)
//   detailed analysis only of diff pixels!
//
// 100% * distance, 0 is best fit
// = similarity of 2 chars for recognition of noisy chars
//   weigth of pixels with only one same neighbour set to 0
//   look at contours too!
int distance2( pix *p1, struct box *box1,
               pix *p2, struct box *box2, int cs){
   int rc=0,x,y,v1,v2,i1,i2,rgood=0,rbad=0,
       x1,y1,x2,y2,dx,dy,dx1,dy1,dx2,dy2,tx,ty;
   x1=box1->x0;y1=box1->y0;x2=box2->x0;y2=box2->y0;
   dx1=box1->x1-box1->x0+1; dx2=box2->x1-box2->x0+1; dx=((dx1>dx2)?dx1:dx2);dx=dx1;
   dy1=box1->y1-box1->y0+1; dy2=box2->y1-box2->y0+1; dy=((dy1>dy2)?dy1:dy2);dy=dy1;
   if(abs(dx1-dx2)>1+dx/16 || abs(dy1-dy2)>1+dy/16) rbad++; // how to weight?
   // compare relations to baseline and upper line
   if(box1->m4>0 && box2->m4>0){  // used ???
     if(2*box1->y1>box1->m3+box1->m4 && 2*box2->y1<box2->m3+box2->m4) rbad+=128;
     if(2*box1->y0>box1->m1+box1->m2 && 2*box2->y0<box2->m1+box2->m2) rbad+=128;
   }
   tx=dx/16; if(dx<17)tx=1; // raster
   ty=dy/32; if(dy<33)ty=1;
   // compare pixels
   for( y=0;y<dy;y+=ty )
   for( x=0;x<dx;x+=tx ) {	// try global shift too ???
     v1=((pixel(p1,x1+x*dx1/dx,y1+y*dy1/dy)<cs)?1:0); i1=8;	// better gray?
     v2=((pixel(p2,x2+x*dx2/dx,y2+y*dy2/dy)<cs)?1:0); i2=8;	// better gray?
     if(v1==v2) { rgood+=16; continue; } // all things are right!
     // what about different pixel???
     // test overlapp of surounding pixels ???
     v1=1;
     v1=-1;
     for(i1=-1;i1<2;i1++)
     for(i2=-1;i2<2;i2++)if(i1!=0 || i2!=0){
       if( ((pixel(p1,x1+x*dx1/dx+i1*(1+dx1/32),y1+y*dy1/dy+i2*(1+dy1/32))<cs)?1:0)
         !=((pixel(p2,x2+x*dx2/dx+i1*(1+dx2/32),y2+y*dy2/dy+i2*(1+dy2/32))<cs)?1:0) ) v1++;
     }
     if(v1>0)
     rbad+=16*v1;
   }
   if(rgood+rbad) rc= 100*rbad/(rgood+rbad); else rc=99;
   if(/* rc<10 && */ env.vvv&1024){
#define DEBUG 2
#if DEBUG == 2
     fprintf(stderr," distance2 rc=%d rgood=%d rbad=%d\n",rc,rgood,rbad);
     out_b(p1,box1->x0,box1->y0,box1->x1-box1->x0+1,
                                box1->y1-box1->y0+1,cs);
     out_b(p2,box2->x0,box2->y0,box2->x1-box2->x0+1,
                                box2->y1-box2->y0+1,cs);
//     out_x(box1);
//     out_x(box2);
#endif
   }
   return rc;
}


char ocr_db(struct box *box1) {
  int d, dist = 1000;
  char c = '_';
  Box *box2, *box3, *head;
  box3 = head = (Box *)list_get_header(&boxlist);

  for_each_data(&boxlist) {
    box2 = (Box *)list_get_current(&boxlist);
    /* do preselect!!! distance() slowly */
    d = distance2( box2->p, box2, head->p, box1, env.cs);
    if (d < dist) {
      dist = d;
      c = box2->c;
      box3 = box2;
    }
  } end_for_each(&boxlist);
  if (env.vvv)
    fprintf(stderr, "\n# db dist=%4d c=%c", dist, c);
  if (dist > 30)
    c = '_';
  else {
    env.vvv |= 1024;		// verbose output for tests
    // d=distance2(*(box3->p),box3,*(box1->p),box1,env.cs);
    env.vvv &= ~1024;
  }
  return c;
}
