/*
This is a Optical-Character-Recognition program
Copyright (C) 2000  Joerg Schulenburg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 Joerg.Schulenburg@physik.uni-magdeburg.de
*/

#include <stdlib.h>
#include <stdio.h>
#include "pgm2asc.h"
#include "gocr.h"

extern List linelist;

const char *getTextLine (int line) {
  int i;
  Element *elem;
  
  if (line < 0 || line > list_total(&linelist))
    return NULL;

  for ( i = 0, elem = linelist.header; i < line && elem != NULL; i++ )
    elem = elem->next;

  if ( elem != NULL )
    return (const char *)elem->data;

  return NULL;
}

void free_textlines(void) {
  for_each_data(&linelist) {
    if (list_get_current(&linelist))
      free(list_get_current(&linelist));
  } end_for_each(&linelist);
  list_free(&linelist);
}

/* collect all the chars from the box tree and write them to a string buffer
   mo is the mode: mode&8 means, use chars even if unsure recognized
 */
void store_boxtree_lines(int mo) {
  char *buffer,*t;	/* temp buffer for text */
  int i = 0;
  unsigned int len = 1024;
  struct box *box2;

  t = buffer = (char *)malloc(len*sizeof(char));
  if ( !buffer ) { 
    fprintf(stderr,"malloc failed!\n"); 
    return;
  }

  if ( env.vvv&1 ) 
    fprintf(stderr,"# store boxtree to lines ...");

  for_each_data(&boxlist) {
    box2 = (struct box *)list_get_current(&boxlist);
    if (box2->c == '\n') {
      list_app( &linelist, (void *)strdup(buffer) );
      memset(buffer, '\0', len);
      t = buffer;
    }
    else if (box2->c == ' ')	// fill large gaps with spaces
    {
      for (i = (box2->x1 - box2->x0) / (2 * env.avX) + 1; i > 0; i--) {
      	*t++ = ' ';
      	if (strlen(buffer) == len) {
	  char *tmp;
	  len = (int)(1.1*len);		/* increase 10% */
	  tmp = (char *)realloc(buffer, len*sizeof(char));
          if( !tmp ) fprintf(stderr,"realloc failed!\n");
	  else buffer = tmp;
	  t = buffer + strlen(buffer);
        }
      }
    }
    else {
      if (box2->c != '_')
	*t++ = box2->c;
      else if (!(mo & 8))
	*t++ = box2->ac;	// alternate char
      else
	*t++ = '_';
      if (strlen(buffer) == len) {
	char *tmp;
	len = (int)(1.1*len);		/* increase 10% */
	tmp = (char *)realloc(buffer, len*sizeof(char));
        if( !tmp ){ fprintf(stderr,"realloc failed!\n"); }
	else buffer = tmp;
        t = buffer + strlen(buffer);
      }
    }
    i++;
  } end_for_each(&boxlist);

  /* do not forget last line */
  // is there no \n in the last line? If there is, delete next line.
  list_app( &linelist, (void *)strdup(buffer) );
  free(buffer);
  if( env.vvv&1 ) fprintf(stderr,"... %d lines\n",i);
}
